/*
 * Copyright (c) 1996 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */


#include <TopoFace/fgeomobjs.h>
#include <DemVtk/demtriangle.h>
#include "vtkPolygon.h"
#include "vtkPlane.h"
#include "vtkMath.h"
#include "vtkLine.h"
#include <stdio.h>

class DemPlane : public vtkPlane {
public:
  DemPlane();

  float EvaluateZ(float normal[3], float origin[3], float x[3]);
};

DemPlane::DemPlane() : vtkPlane() {}

float DemPlane::EvaluateZ(float normal[3], float origin[3], float x[3])
{
  return (( -normal[0]*(x[0]-origin[0]) - 
           normal[1]*(x[1]-origin[1]) ) / normal[2] ) + origin[2];
}

static vtkPolygon poly;
static DemPlane plane;

DemTriangle::DemTriangle() : vtkTriangle() {}
DemTriangle::DemTriangle(const vtkTriangle& t) : vtkTriangle(t) {}

float DemTriangle::XYDistance2BetweenPoints(float x[3], float y[3]) {
  return ((x[0]-y[0])*(x[0]-y[0]) + (x[1]-y[1])*(x[1]-y[1]));
}

float DemTriangle::EvaluateZ(float x[3]) {
  float *pt1, *pt2, *pt3, n[3];
  //  printf("x: %f, y: %f\n", x[0], x[1]);
  pt1 = this->Points.GetPoint(1);
  pt2 = this->Points.GetPoint(2);
  pt3 = this->Points.GetPoint(0);
  //  printf("pt1: %f, %f, %f\n", pt1[0], pt1[1], pt1[2]);
  //  printf("pt2: %f, %f, %f\n", pt2[0], pt2[1], pt2[2]);
  //  printf("pt3: %f, %f, %f\n", pt3[0], pt3[1], pt3[2]);
  vtkFloatPoints* threepts = vtkFloatPoints::New();
  threepts->Allocate(3);
  threepts->InsertNextPoint(pt1);
  threepts->InsertNextPoint(pt2);
  threepts->InsertNextPoint(pt3);
  poly.ComputeNormal (threepts, n);
  threepts->Delete();
  //  printf("normal: %f, %f, %f\n", n[0], n[1], n[2]);
  float z = plane.EvaluateZ(n, pt1, x);
  //  printf("z: %f\n", z);
  return z;
}

int DemTriangle::EvaluatePosition(float x[3], float closestPoint[3],
                                 int& subId, float pcoords[3], 
                                 float& dist2, float *weights)
{
  int i, j;
  float *pt1, *pt2, *pt3, n[3];
  float rhs[2], c1[2], c2[2];
  float det;
  float maxComponent;
  int idx=0, indices[2];
  float fx[3], fy[3];

  subId = 0;
  pcoords[0] = pcoords[1] = pcoords[2] = 0.0;

  pt1 = this->Points.GetPoint(1);
  pt2 = this->Points.GetPoint(2);
  pt3 = this->Points.GetPoint(0);

  fx[0] = pt1[0];
  fx[1] = pt2[0];
  fx[2] = pt3[0];
  fy[0] = pt1[1];
  fy[1] = pt2[1];
  fy[2] = pt3[1];
  //  printf("%f %f\n%f %f\n%f %f\n%f %f\n\n", fx[0], fy[0], fx[1], fy[1],
  //  	 fx[2], fy[2], fx[0], fy[0]);
  //  printf("#probe point\n%f %f\n\n", x[0], x[1]);
  FFillPolygonObj ft(fx, fy, 3);
  FPointObj fpt(x[0], x[1]);
  int inside = ft.Contains(fpt);
  //  if (inside)
  //    printf("#is inside\n");
  //  else
  //    printf("#is not inside\n");
  if (inside) {
//
// Get normal for triangle
//
  vtkFloatPoints* threepts = vtkFloatPoints::New();
  threepts->Allocate(3);
  threepts->InsertNextPoint(pt1);
  threepts->InsertNextPoint(pt2);
  threepts->InsertNextPoint(pt3);
  poly.ComputeNormal (threepts, n);
  threepts->Delete();
//
// Project point to plane
//
  plane.ProjectPoint(x,pt1,n,closestPoint);
    dist2 = XYDistance2BetweenPoints(closestPoint,x); //projection distance
  }
  return inside == 1 ? 1 : -1;
}
