/*
 * Copyright 1996 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems, Cider Press
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapdialogs.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstates.h>
#include <MapUnidraw/mapviewer.h>

#include <Map/mapdb.h>
#include <Map/mapcontour.h>
#include <Map/mapcov.h>
#include <Map/mapfclass.h>
#include <Map/mapfeature.h>
#include <Map/maplib.h>

#include <OverlayUnidraw/ovdialog.h>
#include <OverlayUnidraw/ovpage.h>

#include <IVGlyph/gdialogs.h>
#include <IVGlyph/ofilechooser.h>
#include <IV-look/kit.h>

#include <Unidraw/Commands/edit.h>
#include <Unidraw/Graphic/ellipses.h>
#include <Unidraw/Graphic/graphic.h>
#include <Unidraw/Graphic/polygons.h>
#include <Unidraw/Graphic/pspaint.h>
#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/globals.h>
#include <Unidraw/iterator.h>
#include <Unidraw/selection.h>
#include <Unidraw/viewer.h>
#include <Unidraw/unidraw.h>

#include <InterViews/cursor.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>
#include <InterViews/perspective.h>
#include <InterViews/style.h>
#include <OS/math.h>

#ifdef HAVE_ACE
#include <ace/SOCK_Connector.h>
#include <ace/Log_Msg.h>
#include <ace/Log_Record.h>
#endif

#include <limits.h>
#include <string.h>
#include <sys/stat.h>
#include <stdio.h>
extern "C" {
#include <unistd.h>
}

/*****************************************************************************/

#if 0
OpenDemViewCmd::OpenDemViewCmd(ControlInfo* ci) : Command(ci) {}
OpenDemViewCmd::OpenDemViewCmd(Editor* ed) : Command(ed) {}

void OpenDemViewCmd::Execute() {
  MapEditor* maped = (MapEditor*)GetEditor();
  if (maped && !maped->GetVtkEditor()) {
    VtkEditor* ve = new VtkEditor((const char*)nil);
    maped->SetVtkEditor(ve);
    unidraw->Open(ve);
  }
}
#endif

/*****************************************************************************/

#ifdef HAVE_ACE
ACE_SOCK_Stream* SendCameraCmd::_socket = nil;
#endif

SendCameraCmd::SendCameraCmd(float px, float py, float fx, float fy) {
    _pos.setposition(px, py, fx, fy);
}

#if 0
SendCameraCmd::~SendCameraCmd() {
}
#endif

void SendCameraCmd::Execute() {
#ifdef HAVE_ACE
    if (!_socket) {
        Catalog* catalog = unidraw->GetCatalog();
        const char* portstr = catalog->GetAttribute("camport");
        u_short CAMERA_PORT = atoi(portstr);
        const char *const CAMERA_HOST = catalog->GetAttribute("host");
        const char *camera_host = CAMERA_HOST;
        u_short camera_port  = CAMERA_PORT;


        _socket = new ACE_SOCK_Stream;
	ACE_SOCK_Connector* connector = new ACE_SOCK_Connector;
        ACE_INET_Addr* addr = new ACE_INET_Addr(camera_port, camera_host);

        if (connector->connect (*_socket, *addr) == -1) {
	  ACE_ERROR ((LM_ERROR, "%p\n", "open"));
	  WidgetKit& kit = *WidgetKit::instance();
	  Style* ts = new Style(kit.style());
	  ts->attribute("name", "CameraPos");
	  kit.push_style(ts);
	  GAcknowledgeDialog* dialog = new GAcknowledgeDialog
	    ("Could not connect to camera service");
	  Resource::ref(dialog);
	  dialog->post_for(GetEditor()->GetWindow());
	  Resource::unref(dialog);
	  kit.pop_style();
	  delete _socket;
	  _socket = nil;
	  return;
	}
    }
    if (_socket->send (&_pos, sizeof(_pos)) == -1) {
          ACE_ERROR ((LM_ERROR, "%p\n", "send"));
	  WidgetKit& kit = *WidgetKit::instance();
	  Style* ts = new Style(kit.style());
	  ts->attribute("name", "CameraPos");
	  kit.push_style(ts);
	  GAcknowledgeDialog* dialog = new GAcknowledgeDialog
	    ("Error sending camera command");
	  Resource::ref(dialog);
	  dialog->post_for(GetEditor()->GetWindow());
	  Resource::unref(dialog);
	  kit.pop_style();
    }
#endif
}

/*****************************************************************************/

ClassId MapOpenCmd::GetClassId () { return MAPOPEN_CMD; }

boolean MapOpenCmd::IsA (ClassId id) {
    return MAPOPEN_CMD == id || OvOpenCmd::IsA(id);
}

MapOpenCmd::MapOpenCmd (ControlInfo* c, OpenFileChooser* fc) : OvOpenCmd(c, fc) { }
MapOpenCmd::MapOpenCmd (Editor* ed, OpenFileChooser* fc) : OvOpenCmd(ed, fc) { }

Command* MapOpenCmd::Copy () {
    Command* copy = new MapOpenCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void MapOpenCmd::Execute () {
    MapEditor* ed = (MapEditor*)GetEditor();
    OvOpenCmd::Execute();
    MapIdrawComp* comp =  (MapIdrawComp*)ed->GetComponent();

    //single database case
    Iterator i;
    comp->First(i);    
    MapDbComp* dbcomp = (MapDbComp*)comp->GetComp(i);
    ed->SetMapDbComp(dbcomp);
   
    ed->OpenInit();

    MapShowAllCmd* showall = new MapShowAllCmd(ed);
    showall->Execute();
}

/*****************************************************************************/

ClassId MapShowCmd::GetClassId () { return MAPSHOW_CMD; }

boolean MapShowCmd::IsA (ClassId id) {
    return MAPSHOW_CMD == id || Command::IsA(id);
}

MapShowCmd::MapShowCmd (ControlInfo* c, boolean use_editor, boolean hidden)
: Command(c)
{
    _hidden = hidden;
    _use_ed = use_editor;
    _libname = _covname = _fcname = nil;
    _mag = -1.;
    _proj = nil;
    _tileid = 1;
}

MapShowCmd::MapShowCmd (Editor* ed, boolean use_editor, boolean hidden)
: Command(ed)
{
    _hidden = hidden;
    _use_ed = use_editor;
    _libname = _covname = _fcname = nil;
    _mag = -1.;
    _proj = nil;
    _tileid = 1;
}

Command* MapShowCmd::Copy () {
    Command* copy = new MapShowCmd(CopyControlInfo(), _use_ed);
    InitCopy(copy);
    ((MapShowCmd*)copy)->SetMag(_mag);
    ((MapShowCmd*)copy)->SetProjection(_proj);
    return copy;
}

void MapShowCmd::SetLibName(const char* ln) {
    _libname = (char*)ln;
}

void MapShowCmd::SetCovName(const char* cn) {
    _covname = (char*)cn;
}

void MapShowCmd::SetFClassName(const char* fcn) {
    _fcname = (char*)fcn;
}

void MapShowCmd::SetUseEditor(boolean ue) {
    _use_ed = ue;
}

void MapShowCmd::SetHidden(boolean h) {
    _hidden = h;
}

void MapShowCmd::SetMag(float m) {
    _mag = m;
}

void MapShowCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    if (_use_ed) {
	_libname = (char*)editor->libstate()->maplib();
	_covname = (char*)editor->covstate()->mapcov();
	_fcname = (char*)editor->fclassstate()->mapfclass();
    }
    if (!_libname) return;
    if (!_covname) return;
    if (!_fcname) return;

    editor->GetWindow()->cursor(hourglass);
    editor->GetMapDbComp()->Interpret(this);
    if (_mag > 0.0)
      editor->GetViewer()->SetMagnification(_mag);
    unidraw->Update();
}

boolean MapShowCmd::Reversible () { return false; }

/*****************************************************************************/

ClassId MapShowAllCmd::GetClassId () { return MAPSHOWALL_CMD; }

boolean MapShowAllCmd::IsA (ClassId id) {
    return MAPSHOWALL_CMD == id || MapShowCmd::IsA(id);
}

MapShowAllCmd::MapShowAllCmd (ControlInfo* c, boolean use_editor, boolean hidden)
: MapShowCmd(c, use_editor, hidden) {
}

MapShowAllCmd::MapShowAllCmd (Editor* ed, boolean use_editor, boolean hidden)
: MapShowCmd(ed, use_editor, hidden) {
}

Command* MapShowAllCmd::Copy () {
    Command* copy = new MapShowAllCmd(CopyControlInfo(), _use_ed);
    InitCopy(copy);
    return copy;
}

void MapShowAllCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    editor->GetWindow()->cursor(hourglass);
    editor->GetMapDbComp()->Interpret(this);
    if (_mag > 0.0)
      editor->GetViewer()->SetMagnification(_mag);
    unidraw->Update();
}

/*****************************************************************************/

ClassId MapHideCmd::GetClassId () { return MAPHIDE_CMD; }

boolean MapHideCmd::IsA (ClassId id) {
    return MAPHIDE_CMD == id || Command::IsA(id);
}

MapHideCmd::MapHideCmd (ControlInfo* c, boolean use_editor)
: Command(c)
{
    _use_ed = use_editor;
    _libname = _covname = _fcname = nil;
}

MapHideCmd::MapHideCmd (Editor* ed, boolean use_editor)
: Command(ed)
{
    _use_ed = use_editor;
    _libname = _covname = _fcname = nil;
}

Command* MapHideCmd::Copy () {
    Command* copy = new MapHideCmd(CopyControlInfo(), _use_ed);
    InitCopy(copy);
    return copy;
}

void MapHideCmd::SetLibName(const char* ln) {
    _libname = (char*)ln;
}

void MapHideCmd::SetCovName(const char* cn) {
    _covname = (char*)cn;
}

void MapHideCmd::SetFClassName(const char* fcn) {
    _fcname = (char*)fcn;
}

void MapHideCmd::SetUseEditor(boolean ue) {
    _use_ed = ue;
}

void MapHideCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    if (_use_ed) {
	_libname = (char*)editor->libstate()->maplib();
	_covname = (char*)editor->covstate()->mapcov();
	_fcname = (char*)editor->fclassstate()->mapfclass();
    }
    if (!_libname) return;
    if (!_covname) return;
    if (!_fcname) return;

    editor->GetWindow()->cursor(hourglass);
    editor->GetMapDbComp()->Interpret(this);
    unidraw->Update();
}

boolean MapHideCmd::Reversible () { return false; }

/*****************************************************************************/

MapCenterCmd::MapCenterCmd(ControlInfo* ci) : Command(ci) {}
MapCenterCmd::MapCenterCmd(Editor* ed) : Command(ed) {}

void MapCenterCmd::CenterLonLat(float lon, float lat) {
  _lon = lon;
  _lat = lat;
}

void MapCenterCmd::Execute() {
  float dx, dy, fx, fy;
  int sx, sy;
  MapViewer* viewer = (MapViewer*)GetEditor()->GetViewer();
  viewer->GeoToScreen(_lon, _lat, sx, sy);
  viewer->CenterToScreen(sx, sy);
}

/*****************************************************************************/

ClassId ContourCmd::GetClassId () { return CONTOUR_CMD; }

boolean ContourCmd::IsA (ClassId id) {
    return CONTOUR_CMD == id || Command::IsA(id);
}

ContourCmd::ContourCmd (ControlInfo* c) : Command(c) { }
ContourCmd::ContourCmd (Editor* ed) : Command(ed) { }

Command* ContourCmd::Copy () {
    Command* copy = new ContourCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void ContourCmd::Execute () {
    Editor* editor = GetEditor();
    Catalog* catalog = unidraw->GetCatalog();
    char* contdir = strdup(catalog->GetAttribute("contour"));
    struct stat status;
    if (strlen(contdir) > 0 && stat(contdir, &status) == 0) {
	ContourFeatureClass* fc = new ContourFeatureClass(contdir);
	Clipboard* clb = new Clipboard();
	for (int i = 0; i < fc->nfeats(); i++) {
	    MapFeature* vf = fc->feature(i);
	    MapFeatureComp* mfc = new MapFeatureComp(vf);
	    ((ContourFeature*)vf)->graphic(mfc);
	    clb->Append(mfc);
	}
	PasteCmd* pastecmd = new PasteCmd(editor, clb);
	pastecmd->Execute();
	Selection* s = GetEditor()->GetViewer()->GetSelection();
	s->Clear();
    }
    delete contdir;
}

boolean ContourCmd::Reversible () { return true; }

/*****************************************************************************/

StartBlinkingCmd::StartBlinkingCmd (ControlInfo* c) : Command(c) { }
StartBlinkingCmd::StartBlinkingCmd (Editor* ed) : Command(ed) { }

Command* StartBlinkingCmd::Copy () {
    Command* copy = new StartBlinkingCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void StartBlinkingCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    editor->ProjNameView()->blink_view();
}

/*****************************************************************************/

StopBlinkingCmd::StopBlinkingCmd (ControlInfo* c) : Command(c) { }
StopBlinkingCmd::StopBlinkingCmd (Editor* ed) : Command(ed) { }

Command* StopBlinkingCmd::Copy () {
    Command* copy = new StopBlinkingCmd(CopyControlInfo());
    InitCopy(copy);
    return copy;
}

void StopBlinkingCmd::Execute () {
    MapEditor* editor = (MapEditor*)GetEditor();
    editor->ProjNameView()->stop_blinking();
}
