/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <VpfUnidraw/vpfedtile.h>
#include <VpfUnidraw/vpfclasses.h>
#include <VpfUnidraw/vpfcmds.h>
#include <VpfUnidraw/vpfeditor.h>
#include <VpfUnidraw/vpfviewstate.h>

#include <Vpf/vpfcov.h>
#include <Vpf/vpfdb.h>
#include <Vpf/vpffclass.h>
#include <Vpf/vpflib.h>

#include <MapUnidraw/mapclasses.h>
#include <MapUnidraw/mapcmds.h>
#include <MapUnidraw/mapinspect.h>
#include <MapUnidraw/mapproj.h>
#include <MapUnidraw/mapstatecmds.h>
#include <MapUnidraw/mapstates.h>

#include <Map/mapelt.h>

#include <OverlayUnidraw/ovviewer.h>

#include <UniIdraw/idvars.h>

#include <Unidraw/catalog.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/iterator.h>
#include <Unidraw/keymap.h>
#include <Unidraw/kybd.h>
#include <Unidraw/selection.h>
#include <Unidraw/ulist.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Commands/edit.h>
#include <Unidraw/Graphic/ellipses.h>
#include <Unidraw/Graphic/lines.h>
#include <Unidraw/Graphic/polygons.h>

#include <InterViews/border.h>
#include <InterViews/box.h>
#include <InterViews/cursor.h>
#include <InterViews/frame.h>
#include <InterViews/glue.h>
#include <InterViews/menu.h>
#include <InterViews/message.h>
#include <InterViews/panner.h>
#include <InterViews/tray.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

#include <string.h>

VpfTileEditor::VpfTileEditor(int n, VpfFeature** t, VpfEditor* vie,
			     OverlayKit* ok) 
: VpfEditor(false, ok)
{
    _tiles = t;
    _ntiles = n;
    _parent = vie;
    _mapdb = vie->GetMapDatabase();
    _projection = nil;
    Init(nil, "VpfTileEditor");

    Initializing(true);
}

VpfTileEditor::VpfTileEditor(boolean initflag, OverlayKit* ok) : VpfEditor(initflag, ok) {
}

void VpfTileEditor::InitStateVars () {
    IdrawEditor::InitStateVars();
}

void VpfTileEditor::Init (OverlayComp* comp, const char* name) {
    _overlay_kit->Init(comp, name);
}

void VpfTileEditor::InitCommands() {
    MapProjectionCmd* pcmd = new MapProjectionCmd(this);
    pcmd->SetProjection(_parent->projstate()->projection());
    MapShowCmd* show = new MapShowCmd(this);
    show->Execute();
}

VpfFeature* VpfTileEditor::tile(int n) {
    if (n >= 0 && n < _ntiles)
	return _tiles[n];
    else
	return nil;
}

int VpfTileEditor::ntiles() {
    return _ntiles;
}
